@@ -1,6 +1,6 @@
 =head1 NAME
 
-SQL::Steno - short hand for SQL and compact output
+SQL::Steno - Short hand for SQL and compact output
 
 =head1 SYNOPSIS
 
@@ -10,18 +10,20 @@ Type some short-hand, see the corresponding SQL and its output:
  select * from TABLE1 where somecolumn > 2;
  prepare: 0.000s   execute: 0.073s   rows: 14
    id|column1                                    |column2
-     |                                           |    |somecolumn
- ----|-------------------------------------------|----|-|
-   27|foo                                        |    |7|
-   49|bar                                        |abcd|3|
-   81|baz\nbazinga\nbazurka                      |jk  |9|
- 1984|bla bla bla bla bla bla bla bla bla bla bla|xyz |5|
+     |                                           |    |column3
+     |                                           |    | |somecolumn
+ ----|-------------------------------------------|----|-|-|
+   27|foo                                        |    |a|7|
+   49|bar                                        |abcd|a|3|
+   81|baz\nbazinga\nbazurka                      |jk  |b|9|
+ 1984|bla bla bla bla bla bla bla bla bla bla bla|xyz |c|5|
  ...
  steno> /abc|foo/#TBL1;.socol > 2    -- /regexp/ grep, #tableabbrev, .columnabbrev
  select * from TABLE1 where somecolumn > 2;
  prepare: 0.000s   execute: 0.039s   rows: 14
  id|column1
    |   |column2
+   |   |    |[column3=a]
    |   |    |somecolumn
  --|---|----|-|
  27|foo|    |7|
@@ -29,6 +31,9 @@ Type some short-hand, see the corresponding SQL and its output:
  steno> .c1,.c2,.some;#TE1#:ob2d3    -- ; before tables means from, 2nd # alias, :macro
  select column1,column2,somecolumn from TABLE1 TE1 order by 2 desc, 3;
  ...
+ steno> n(), yr(), cw(,1,2,3)    -- functionabbrev before (, can have initial default arg
+ select count(*), year(now()), concat_ws(',',1,2,3);
+ ...
  steno> .col1,.clm2,.sn;#TBL1:jTBL2 u(id);mydate :b :m+3d and :d-w    -- :jTABLEABBREV and :+/- family
  select column1,column2,somecolumn from TABLE1 join TABLE2 using(id) where mydate between date_format(now(),"%Y-%m-01")+interval 3 day and curdate()-interval 1 week;
  ...
@@ -60,7 +65,7 @@ with other DBs.
 use v5.10;
 
 package SQL::Steno;
-our $VERSION = v0.1;
+our $VERSION = 'v0.2';	# quotes because of CPAN indexer bug
 
 use strict;
 use Time::HiRes qw(gettimeofday tv_interval);
@@ -69,7 +74,7 @@ binmode STDIN, ':utf8';
 binmode STDOUT, ':utf8';
 
 our $dbh;
-
+our $perl_re = qr/(\{(?:(?>[^{}]+)|(?-1))*\})/;
 our( %Table_Columns, $table_re );
 sub init {
     die "\$dbh is undef\n" unless $dbh;
@@ -108,6 +113,10 @@ sub Queries(@) {
     }
 }
 Queries
+    ['.csv' => '   output "&.csv() this" or next query as CSV',
+      '&{ $print_handler = \&print_csv; "" }'],
+
+
     [ps => '   show processlist (without Sleep)',
       '/^(?!.+\|Sleep)/=show processlist'],
 
@@ -153,6 +162,8 @@ our %Macros =
     nl => ' not like',
     nn => ' is not null',
     nr => ' not rlike',
+    od => ' on duplicate key update',
+    odku => ' on duplicate key update',
     r => ' rlike',
     u => ' union select',
     ua => ' union all select',
@@ -170,7 +181,6 @@ INT
 	    unless @_;		# help
 	for( $_[0] ) {
 	    return " limit $_" if /^\d+$/;
-	    # \todo [og]b
 	    if( s/^([og])b(?=(?:\d[ad]?)*$)/ $1 eq 'g' ? ' group by ' : ' order by ' /e ) {
 		s/(?<! )(?=\d)/, /g;
 		s/a/ asc/g; s/(?<!r)d/ desc/g;
@@ -226,32 +236,45 @@ our %Functions =
     u => 'using',		# -"-
     uc => 'ucase'
    );
-our @Functions = qw(
-    abs acos adddate addtime aes_decrypt aes_encrypt ascii asin atan atan avg
-    benchmark bin bit_and bit_count bit_length bit_or bit_xor cast ceiling
-    char_length char character_length charset coalesce coercibility collation
-    compress concat_ws concat connection_id conv convert_tz cos cot count crc32
-    curdate current_date current_time current_timestamp current_user curtime
-    database date_add date_format date_sub date datediff day dayname dayofmonth
-    dayofweek dayofyear decode default degrees des_decrypt des_encrypt elt encode
-    encrypt exp export_set field find_in_set floor format found_rows from_days
-    from_unixtime get_format get_lock greatest group_concat hex hour if ifnull
-    inet_aton inet_ntoa insert instr interval is_free_lock is_used_lock isnull
-    last_insert_id lcase least left length ln load_file localtime localtimestamp
-    locate log10 log2 log lower lpad ltrim make_set makedate maketime
-    master_pos_wait max md5 microsecond mid min minute mod month monthname
-    name_const now nullif oct octet_length old_password ord password period_add
-    period_diff pi position power quarter quote radians rand release_lock repeat
-    replace reverse right round row_count rpad rtrim schema sec_to_time second
-    session_user sha1 sign sin sleep soundex space sqrt stddev stddev_pop
-    stddev_samp str_to_date strcmp subdate substring_index substring subtime sum
-    sysdate system_user tan time_format time_to_sec time timediff timestamp
-    timestampadd timestampdiff to_days trim truncate ucase uncompress
-    uncompressed_length unhex unix_timestamp upper user utc_date utc_time
-    utc_timestamp uuid values var_pop var_samp variance week weekday weekofyear
-    year yearweek);
 
-# \todo #tbl -> table  #tbl# -> table tbl  #tbl#alias -> table alias  :jtbl#alias -> join table alias using...
+# functions where the 1st argument can be now()
+my @nowFunctions = qw(
+    adddate addtime convert_tz date date_add date_format date_sub datediff day
+    dayname dayofmonth dayofweek dayofyear hour last_day minute month
+    monthname quarter second subdate subtime time time_format time_to_sec
+    timediff timestamp to_days to_seconds week weekday weekofyear year
+    yearweek
+);
+our @Functions = sort @nowFunctions, qw(
+    abs acos aes_decrypt aes_encrypt ascii asin atan avg benchmark bin bit_and
+    bit_count bit_length bit_or bit_xor cast ceiling char_length char
+    character_length charset coalesce coercibility collation compress
+    concat_ws concat connection_id conv cos cot count crc32 curdate
+    current_date current_time current_timestamp current_user curtime database
+    decode default degrees des_decrypt des_encrypt elt encode encrypt exp
+    export_set field find_in_set floor format found_rows from_days
+    from_unixtime get_format get_lock greatest group_concat hex if ifnull
+    inet_aton inet_ntoa insert instr interval is_free_lock is_used_lock isnull
+    last_insert_id lcase least left length ln load_file localtime
+    localtimestamp locate log10 log2 log lower lpad ltrim make_set makedate
+    maketime master_pos_wait max md5 microsecond mid min mod name_const now
+    nullif oct octet_length old_password ord password period_add period_diff
+    pi position power quote radians rand release_lock repeat replace reverse
+    right round row_count rpad rtrim schema sec_to_time session_user sha1 sign
+    sin sleep soundex space sqrt stddev stddev_pop stddev_samp str_to_date
+    strcmp substring_index substring sum sysdate system_user tan timestampadd
+    timestampdiff trim truncate ucase uncompress uncompressed_length unhex
+    unix_timestamp upper user utc_date utc_time utc_timestamp uuid values
+    var_pop var_samp variance
+);
+
+our %DefaultArguments = (
+    count => '*',
+    concat_ws => "','"
+);
+$DefaultArguments{$_} = 'now()' for @nowFunctions;
+
+
 our %Tables;
 our %Columns;
 
@@ -332,7 +355,7 @@ sub convert_table_column {
 	    undef $column{$_} for @{$Table_Columns{$_}};
 	}
 	my @column = keys %column;
-	s/(^|[-+\s(,;&|])?(?<!\\)\.([a-z]\w*)/(defined $1 ? $1 : '.') . find $2, '.', '', %Columns, @column/egi;
+	s/(^|[-+\s(,;&|])?(?<!\\)\.([a-z]\w*)(?:\.(\w*))?/(defined $1 ? $1 : '.') . (find $2, '.', '', %Columns, @column) . ($3 ? " $3" : defined $3 ? " $2" : '')/egi;
     }
 }
 
@@ -344,11 +367,13 @@ L</shell> for more run time oriented features.
 
 First it looks for C<:macro>.  These are mostly simple text-replacements
 stored in C<%Macros>.  There are also some dynamic macros.  Those starting
-with C<:j> or C<:lj> may continue into a table spec without the leading C<#>.
-E.g. C<:ljtbl#t> might expand to C<left join table t>.
+with C<:j> (join) or C<:lj> (left join) may continue into a table spec without
+the leading C<#>.  E.g. C<:ljtbl#t> might expand to C<left join table t>.
 
-Those starting with C<:gb> or C<:ob> may be followed by result columns numbers
-from 1-9, each optionally followed by a or d for asc or desc.
+Those starting with C<:gb> (group by) or C<:ob> (order by) may be followed by
+result columns numbers from 1-9, each optionally followed by a or d for asc or
+desc.  Macros can also be C<:{Perl code}>, which gets replaced by what it
+returns.
 
 Then there are the time macros, where an optional leading letter indicates a
 base time, and an optional trailing letter with an optional count means the
@@ -392,11 +417,14 @@ offset.  The letters are:
 
 Then it looks for C<#tbl>, C<#tbl#> or C<#tbl#alias>.  Here tbl is a key of
 C<%Tables> or any abbreviation of known tables in C<@Tables>.  If followed by
-C<#>, the abbrev is used as an alias, unless an alias directly follows, in
-which case that is used.
+C<#>, the abbreviation is used as an alias, unless an alias directly follows,
+in which case that is used.
 
-Then it looks for C<.col>.  Here col is a key of C<%Columns> or any
-abbreviation of columns of any table recognized in the query.
+Then it looks for C<.col>, C<.col.> C<.col.alias>.  Here col is a key of
+C<%Columns> or any abbreviation of columns of any table recognized in the
+query.  If followed by C<.>, the abbreviation is used as an alias, unless an
+alias directly follows, in which case that is used.  It tries to be clever
+about whether the 1st C<.> needs to be preserved, i.e. following a table name.
 
 Finally it looks for C<func(>.  Here func is a key of C<%Functions> or any
 abbreviation of known functions in C<@Functions>, which includes words
@@ -405,7 +433,10 @@ C<i(> is C<in(>, so that C<in(> is free for matching C<ifnull(>, but more
 importantly, because it has a smart brother: C<I(> is also C<in(>, where
 quoting is applied as necessary, and unless you give an initial comma C<I(,>,
 the elements are separated on space.  E.g. C<I(a 3 a's q"q)> and
-C<I(,a,3,a's,q"q)> both give C<in("a",3,"a's",'q"q')>.
+C<I(,a,3,a's,q"q)> both give C<in("a",3,"a's",'q"q')>.  If the 1st argument of
+a function is empty and the abbrev or function is found in
+C<%DefaultArguments> the value becomes the 1st argument.
+
 
 Finally it picks on the structure of the statement: These keywords can be
 abbreviated: C<se(lect)>, C<ins(ert)>, C<upd(ate)> or C<del(ete)>.  If none of
@@ -438,14 +469,19 @@ sub convert {
 	}
     }
 
-    # \todo (?(?<=\w)\b)
-    s&(?<!\\):($timespec_re[+-]\d*$timespec_re(?(?<=\w)\b)|l?j\w+(?:#(\w*))|\w+)&find $1, ':', '', %Macros&ego unless $error;
+    until( $error ) {
+	s&:$perl_re&my $ret = eval $1; warn $@ if $@; $ret // ' NULL '&ego or
+	    # \todo (?(?<=\w)\b)
+	    s&:($timespec_re[+-]\d*$timespec_re(?(?<=\w)\b)|l?j\w+(?:#(\w*))|\w+)&find $1, ':', '', %Macros&ego or
+		last;
+    }
 
     s&^(?=#)&;&;		# Assume empty fieldlist before table name
     &convert_table_column;
     s&^(?=$table_re)&;&;	# Assume empty fieldlist before table name
 
-    s&\b(\w+)(?=\()&find $1, '', '(', %Functions, @Functions or $1&eg unless $error;
+    s&\b(\w+)\((?=\s*([,)])?)&my $fn = find $1, '', '(', %Functions, @Functions; ($fn || $1).'('.($2 and $DefaultArguments{$1} || $DefaultArguments{$fn} or '')&eg unless $error;
+    #s&\b(\w+)(?=\()&find $1, '', '(', %Functions, @Functions or $1&eg unless $error;
 
     return if $error;
     s/\A\s*;/*;/;
@@ -472,11 +508,125 @@ my %ctrl =
      "\r", '\r');
 my $lasttime = time;
 
-sub run($$;$$$) {
-    my( $dbh, $sql, $re, $fh, $csv ) = @_;
+sub print_tabular($;$$) {
+    my( $sth, $re, $sub ) = @_;
+    my $filter;			# don't filter names
+    my @name = @{$sth->{NAME}};
+    my @len = (0) x @name;
+    my( @txt, @res, @comp );
+    while( my @res1 = $sth->fetchrow_array ) {
+	next if $sub && $filter && !$sub->( \@res1, \@name );
+	next if $re  && $filter && join( '|', map $_ // '\@', @res1 ) !~ $re;
+	for my $i ( 0..$#res1 ) {
+	    if( !defined $res1[$i] ) {
+		$res1[$i] = '\@';
+	    } elsif( $res1[$i] !~ /^\d+(?:\.\d+)?$/ ) {
+		$txt[$i] = 1;
+		$res1[$i] =~ s/\r\n/\\R/g;
+		$res1[$i] =~ s/([\t\n\r])/$ctrl{$1}/g;
+		no warnings 'uninitialized';
+		$res1[$i] =~ s/^(?:(0000-)00-00|(1970-)01-01)(?:( 00:)00:00)?$/$1$2$3/ or
+		    $res1[$i] =~ s/^(\d{4}-\d\d-\d\d )?(?:00:00:00|23:59:5(9))$/$1 . ($2 ? '24:' : '00:')/e or
+			$res1[$i] =~ s/^((\d{4}-\d\d-\d\d )?\d\d:\d\d):00$/$1/;
+		utf8::decode $res1[$i];
+	    }
+	    $txt[$i] = 0 if @txt < $i;
+	    my $len = length $res1[$i];
+	    $len[$i] = $len if $len[$i] < $len;
+	}
+	if( @comp ) {
+	    for my $i ( 0..$#comp ) {
+		undef $comp[$i] if defined $comp[$i] && $comp[$i] ne $res1[$i];
+	    }
+	} else {
+	    @comp = @res1;
+	}
+	push @res, \@res1;
+    } continue {
+	$filter = 1;
+    }
+    if( @res ) {
+	@comp = () if @res == 1;
+	my $fmt = '';
+	for( my $i = 0; $i < @name; ++$i ) {
+	    $name[$i] =~ s/\r\n/\\R/g;
+	    $name[$i] =~ s/([\t\n\r])/$ctrl{$1}/g;
+	    if( defined $comp[$i] ) {
+		my $more;
+		while( defined $comp[$i] ) {
+		    printf $fmt, @name[0..$i-1] unless $more;
+		    $more = 1;
+		    printf "[%s=%s]", $name[$i], $comp[$i];
+		    @name[0..$i] = ('') x ($i+1);
+		    for my $row ( \@comp, \@name, \@len, \@txt, @res ) {
+			splice @$row, $i, 1;
+		    }
+		}
+		print "\n";
+		--$i, next;
+	    }
+	    if( $len[$i] < length $name[$i] ) {
+		printf "$fmt%s\n", @name[0..$i];
+		@name[0..$i] = ('') x ($i+1);
+	    }
+	    $fmt .= '%' . ($txt[$i] ? -$len[$i] : $len[$i]) . 's|';
+	}
+	$fmt .= "\n";
+	printf $fmt, @name if $name[-1];
+	printf $fmt, map '-'x$_, @len;
+	my $cnt = 0;
+	my $stdout = select eq 'main::STDOUT';
+	my $i =  $stdout ? 100 : ~0;
+	for my $row ( @res ) {
+	    printf $fmt, @$row;
+	    ++$cnt;
+	    if( $stdout && --$i <= 0 && $cnt < @res ) {
+		printf STDERR "How many more, or * for all? (%d of %d) [default: 100] ",
+		    $cnt, scalar @res;
+		$i = <>;
+		if( defined $i ) {
+		    $i =~ tr/ \t\n\r//d;
+		    $i = (0 == length $i) ? 100 :
+			$i eq '*' ? ~0 :
+			    $i == 0 ? last :
+				$i;
+		} else {
+		    print "\n";
+		    last;
+		}
+	    }
+	}
+    }
+}
+sub print_csv($;$$) {
+    my( $sth, $re, $sub ) = @_;
+    my $filter;			# don't filter names
+    my $name = $sth->{NAME};
+    my @row = @$name;
+    while( @row ) {
+	next if $sub && $filter && !$sub->( \@row, $name );
+	next if $re  && $filter && join( '|', map $_ // '\@', @row ) !~ $re;
+	for( @row ) {
+	    $_ //= '';
+	    $_ = qq!"$_"! if
+		s/"/""/g or
+		tr/,\n// or
+		/\A=/;
+	    utf8::decode $_;
+	}
+	print join( ',', @row ) . "\n";
+    } continue {
+	@row = $sth->fetchrow_array;
+	$filter = 1;
+    }
+}
+my %print_handler = (csv => \&print_csv);
+my $print_handler;
+sub run($;$$) {
+    my( $sql, $re, $sub ) = @_;
     my $t0 = [gettimeofday];
-    unless( $t0->[0] - $lasttime < 3600 || $dbh->ping ) {
-	printf STDOUT "Inactive for %ds, ping failed after %.03fs, your set variables are lost.\n",
+    if( $DBI::err || $t0->[0] - $lasttime > 3600 and !$dbh->ping ) {
+	printf STDOUT "Inactive for %ds, ping failed after %.03fs, your session variables are lost.\n",
 	    $t0->[0] - $lasttime, tv_interval $t0;
 	#$dbh->disconnect;
 	$dbh = $dbh->clone;	# reconnect
@@ -490,101 +640,10 @@ sub run($$;$$$) {
 	printf STDOUT "prepare: %.03fs   execute: %.03fs   rows: %d\n",
 	    tv_interval( $t0, $t1 ), tv_interval( $t1 ), $DBI::rows;
 	if( $sth->{Active} ) {
-	    my @name = @{$sth->{NAME}};
-	    print join( ',', @name ) . "\n" if $csv;
-	    my @len = (1) x @name;
-	    my( @txt, @res, @comp );
-	    while( my @res1 = $sth->fetchrow_array ) {
-		next if $re && join( '|', map $_ // '\@', @res1 ) !~ $re;
-		if( $csv ) {
-		    for( @res1 ) {
-			$_ //= '';
-			$_ = qq!"$_"! if
-			    s/"/""/g or
-			    tr/,\n// or
-			    /\A=/;
-			utf8::decode $_;
-		    }
-		    print join( ',', @res1 ) . "\n";
-		    next;
-		} else {
-		    for my $i ( 0..$#res1 ) {
-			if( !defined $res1[$i] ) {
-			    $res1[$i] = '\@';
-			} elsif( $res1[$i] !~ /^\d+(?:\.\d+)?$/ ) {
-			    $txt[$i] = 1;
-			    $res1[$i] =~ s/\r\n/\\R/g;
-			    $res1[$i] =~ s/([\t\n\r])/$ctrl{$1}/g;
-			    no warnings 'uninitialized';
-			    $res1[$i] =~ s/^(?:(0000-)00-00|(1970-)01-01)(?:( 00:)00:00)?$/$1$2$3/ or
-				$res1[$i] =~ s/^(\d{4}-\d\d-\d\d )?(?:00:00:00|23:59:5(9))$/$1 . ($2 ? '24:' : '00:')/e or
-				$res1[$i] =~ s/^((\d{4}-\d\d-\d\d )?\d\d:\d\d):00$/$1/;
-			    utf8::decode $res1[$i];
-			}
-			$txt[$i] = 0 if @txt < $i;
-			my $len = length $res1[$i];
-			$len[$i] = $len if $len[$i] < $len;
-		    }
-		    if( @comp ) {
-			for my $i ( 0..$#comp ) {
-			    undef $comp[$i] if defined $comp[$i] && $comp[$i] ne $res1[$i];
-			}
-		    } else {
-			@comp = @res1;
-		    }
-		    push @res, \@res1;
-		}
-	    }
-	    if( @res ) {
-		@comp = () if @res == 1;
-		my $fmt = '';
-		for( my $i = 0; $i < @name; ++$i ) {
-		    $name[$i] =~ s/\r\n/\\R/g;
-		    $name[$i] =~ s/([\t\n\r])/$ctrl{$1}/g;
-		    if( defined $comp[$i] ) {
-			my $more;
-			while( defined $comp[$i] ) {
-			    printf $fmt, @name[0..$i-1] unless $more;
-			    $more = 1;
-			    printf "[%s=%s]", $name[$i], $comp[$i];
-			    @name[0..$i] = ('') x ($i+1);
-			    for my $row ( \@comp, \@name, \@len, \@txt, @res ) {
-				splice @$row, $i, 1;
-			    }
-			}
-			print "\n";
-			--$i, next;
-		    }
-		    if( $len[$i] < length $name[$i] ) {
-			printf "$fmt%s\n", @name[0..$i];
-			@name[0..$i] = ('') x ($i+1);
-		    }
-		    $fmt .= '%' . ($txt[$i] ? -$len[$i] : $len[$i]) . 's|';
-		}
-		$fmt .= "\n";
-		printf $fmt, @name if $name[-1];
-		printf $fmt, map '-'x$_, @len;
-		my $cnt = 0;
-		my $i = 100;
-		for my $row ( @res ) {
-		    printf $fmt, @$row;
-		    ++$cnt;
-		    if( !$fh && --$i <= 0 && $cnt < @res ) {
-			printf STDERR "How many more, or * for all? (%d of %d) [default: 100] ",
-			    $cnt, scalar @res;
-			$i = <>;
-			if( defined $i ) {
-			    $i =~ tr/ \t\n\r//d;
-			    $i = (0 == length $i) ? 100 :
-				$i eq '*' ? ~0 :
-				$i == 0 ? last :
-				$i;
-			} else {
-			    print "\n";
-			    last;
-			}
-		    }
-		}
+	    if( $print_handler ) {
+		&$print_handler( $sth, $re, $sub );
+	    } else {
+		print_tabular $sth, $re, $sub;
 	    }
 	}
     }
@@ -609,14 +668,30 @@ order (i.e. C<&xyz> can return C</regexp/=literal sql> etc.):
 
 =item &query( arg, ... ) following text
 
+=item &{Perl code} following text
+
 These allow canned entries and are more complex than macros, in that they take
 arguments and replacement can depend on the argument.
 
-=item /regexp/...
+Run I<Perl code>.  It sees the I<following text> in C<$_> and may modify it.
+If it returns undef this statement is skipped.  If it returns a DBI statement
+handle run that instead of this statement.  Else replace with what it returns.
 
-This will treat the statement ... normally, but will join each output row with
-'|' characters and check that against the regexp.  Only matching output rows
-are considered.
+Reprocess result as a shell entry (i.e. it may return another C<&>query).
+
+=item {Perl code}statement
+
+Call I<Perl code> for every output row returned by the I<statement> with the
+array of values as first and the array of column names as second argument.  It
+may modify individual values, but not the array.  If it returns false, the row
+is skipped.
+
+=item /regexp/statement
+
+This will treat the I<statement> normally, but will join each output row with
+'|' characters (which need to be matched as C<\/>) and check that against the
+regexp.  Only matching rows are considered.  You may combine S<C<{Perl
+code}/regexp/>>.
 
 =item =literal sql
 
@@ -635,18 +710,20 @@ matching abbrev or C<?#abbrev> only those tables matching abbrev.
 Will convert and show, but not perform statement.  If C<$dbh> is not set, this
 is the default behaviour.
 
-=item !Unix Shell code
+=item !System Shell code
 
 Run it.
 
-=item {Perl code}
-
-Run it.  If it returns a DBI statement handle also run that.
-
 =item >filename
 
-Redirect next statement to filename.  If it has a suffix C<.csv>, an according
-format is used.
+=item >>filename
+
+Redirect or append next statement's output to I<filename>.  If it has a suffix
+C<.csv>, an according format is used.
+
+=item |System Shell code
+
+Pipe next statement's output through I<System Shell code>.
 
 =back
 
@@ -673,10 +750,33 @@ sub shell() {
 	}
 	s/\s+\Z//;
 
-	s!^\&(\w+)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
-	    or s!^\&(\w+) *(.*)!convert_Queries $1, $2!e unless $error;
+	until( $error ) {
+	    last unless
+		s!^\&(\.?\w+)(\(((?:(?>[^()]+)|(?2))*)\))!convert_Queries $1, $3!e
+		or s!^\&(\.?\w+) *(.*)!convert_Queries $1, $2!e
+		or my $perl = s!^\&$perl_re!!;
+	    if( $perl ) {
+		$perl = eval $1;
+		local $| = 1;	# flush to avoid stderr prompt overtaking last output line.
+		warn $@ if $@;
+		if( UNIVERSAL::isa $perl, 'DBI::st' ) {
+		    $_ = $perl;
+		    goto RUN;
+		} elsif( defined $perl ) {
+		    substr $_, 0, 0, $perl;
+		} else {
+		    goto NEXT;
+		}
+	    }
+	}
 
-	my $re = $1 ? qr/$2/i : qr/$2/ if s!^(i?)/(.+?)/\s*!!;
+	my $sub;
+	if( s/^\s*$perl_re// ) {
+	    $sub = eval "sub $1";
+	    warn $@ if $@;
+	}
+	my $re = $1 ? qr/$2/i : qr/$2/
+	    if s!^(i?)/(.+?)/\s*!!;
 	goto RUN if s/^\s*=//;	# run literally
 
 	my $skip = 0;
@@ -701,20 +801,15 @@ sub shell() {
 	    }
 	    goto NEXT;
 	}
-	if( s/^\s*(\{(?:(?>[^{}]+)|(?1))*\})// ) {
-	    my $ret = eval $1;
-	    warn $@ if $@;
-	    if( UNIVERSAL::isa $ret, 'DBI::st' ) {
-		$_ = $ret;
-		goto RUN;
-	    }
-	    local $| = 1;	# flush to avoid stderr prompt overtaking last output line.
-	    goto NEXT;
-	}
 	s/^\s*()//;			# dummy because $1 survives loop iterations :-o
-	if( /\A>\s*(.+?(\.csv)?)\s*\Z/ ) {	# redirect output
-	    $csv = 1 if $2;
-	    open $fh, '>:utf8', (glob $1)[0];
+	if( /\A(>{1,2})\s*(.+?(?:\.(\w+))?)\s*\Z/ ) {	# redirect output
+	    $print_handler = $print_handler{$3} if $3;
+	    print STDERR "|$print_handler|\n";
+	    open $fh, "$1:utf8", (glob $2)[0];
+	    select $fh;
+	    goto NEXT;
+	} elsif( /\A\|(.+)\Z/ ) {	# pipe output
+	    open $fh, '|-:utf8', $1;
 	    select $fh;
 	    goto NEXT;
 	}
@@ -727,7 +822,8 @@ sub shell() {
 	goto NEXT if $skip;
 
       RUN:
-	run $dbh, $_, $re, $fh, $csv if $dbh;
+	run $_, $re, $sub if $dbh;
+	undef $print_handler;
 	if( $fh ) {
 	    close;
 	    select STDOUT;
@@ -808,10 +904,12 @@ fieldlist defaults to '*', also if Query starts with '#'.
 Abbreviations, more help with ?&{abbrev}, ?#{abbrev}, ?.{abbrev}, ?{abbrev}(, ?:{abbrev}
 &query $1,$2,...	# only at bol
 &query($1,$2,...)...	# only at bol, only replace upto )
+&{Perl code}...		# only at bol, if it return undef then skip, else prepend to ...
 #table
 .column			# for any table recognized in statement
 function(
 :macro
+:{Perl code}		# dynamic macro
 
 Characters \t\n\r get masked in output, \r\n as \R.
 Date or time 0000-00-00 -> 0000-  1970-01-01 -> 1970-  00:00:00 -> 00:  23:59:59 -> 24:
@@ -836,7 +934,7 @@ HELP
     package SQL::Steno;		# doesn't export yet, so get the functions easily
     use SQL::Steno;
     use DBI;
-    our $sql = DBI->connect( ... ); # preferably mysql, but other DBs should work (with limitations).
+    our $dbh = DBI->connect( ... ); # preferably mysql, but other DBs should work (with limitations).
     # If you want #tbl and .col to work, (only) one of:
     init_from_query;		# fast, defaults to mysql information_schema, for which you need read permission
     init;			# slow, using DBI dbh methods.
@@ -850,7 +948,7 @@ the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<DBI>, L<SQL::Interp>, L<SQL::Preproc>, L<SQL::Yapp>
+L<DBI>, L<SQL::Interp>, L<SQL::Preproc>, L<SQL::Yapp>, L<Jade|http://jade-lang.com/>
 
 =head1 AUTHOR
 
